/***************************************************************************//**
 *   @file   AD7980.c
 *   @brief  Implementation of AD7980 Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 699
 *******************************************************************************/
 
 /******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "AD7980.h"	
#include <DSPI.h>

/******************************************************************************/
/************************ Functions Definitions *******************************/
/******************************************************************************/

/***************************************************************************//**
 * @brief Initializes the communication peripheral.
 *
 * @return 1.
*******************************************************************************/
unsigned char AD7980::Init(void)
{
	/*! Initialize SPI communication */
	oDspi.begin(AD7980_CS_PIN);
	oDspi.setMode(DSPI_MODE2);
	oDspi.setSpeed(DEVICE_SPI_CLK_FREQ);
	/*! Initialize class members */
    mVref = 2.5;	/*!< On PmodAD4, Vref = 2.5V */
	
    return(1);
}

/***************************************************************************//**
 * @brief Initiates a conversion and reads data.
 *
 * @return receivedData - Data read from the ADC.
*******************************************************************************/
unsigned short AD7980::ReadRawData(void)
{
    unsigned short receivedData = 0;
    unsigned char rxData[2]    = {0,0};

    oDspi.transfer(0xFF);
	oDspi.transfer(0xFE);
	digitalWrite(AD7980_CS_PIN, LOW);
    rxData[0] = oDspi.transfer(0xFF);
	rxData[1] = oDspi.transfer(0xFF);
    digitalWrite(AD7980_CS_PIN, LOW);
    receivedData = (rxData[0] << 8) + rxData[1];
    
    return(receivedData);
}

/***************************************************************************//**
 * @brief Initiates a conversion and return the value of the input voltage.
 *
 * @return voltage - The value of the input voltage read form the ADC.
*******************************************************************************/
double AD7980::ReadVoltage(void)
{
	unsigned short registerValue = 0;
	double         voltage = 0;
	
	/*! Read raw data from ADC. */
	registerValue = ReadRawData();
	/*! Get the input voltage from the straight binary raw data. */
    voltage = ((double)registerValue / 65536) * mVref;
	
	return(voltage);
}
